@@ -1,8 +1,8 @@
 =begin
 
-= net/http.rb version 1.1.37
+= net/http.rb
 
-Copyright (c) 1999-2001 Yukihiro Matsumoto
+Copyright (c) 1999-2002 Yukihiro Matsumoto
 
 written & maintained by Minero Aoki <aamine@loveruby.net>
 This file is derived from "http-access.rb".
@@ -14,6 +14,8 @@ Ruby Distribute License or GNU General Public License.
 NOTE: You can find Japanese version of this document in
 the doc/net directory of the standard ruby interpreter package.
 
+$Id: http.rb,v 1.41.2.18 2002/02/22 12:50:23 aamine Exp $
+
 == What is this module?
 
 This module provide your program the functions to access WWW
@@ -25,13 +27,11 @@ For details of HTTP, refer [RFC2616]
 
 === Getting Document From Server
 
-Be care to ',' (comma) putted after "response".
-This is required for compatibility.
-
     require 'net/http'
+    Net::HTTP.version_1_1   # declear to use 1.1 features.
     Net::HTTP.start( 'some.www.server', 80 ) {|http|
-      response , = http.get('/index.html')
-      puts response.body
+        response, body = http.get('/index.html')
+        print body
     }
 
 (shorter version)
@@ -42,8 +42,9 @@ This is required for compatibility.
 === Posting Form Data
 
     require 'net/http'
+    Net::HTTP.version_1_1   # declear to use 1.1 features.
     Net::HTTP.start( 'some.www.server', 80 ) {|http|
-        response , = http.post( '/cgi-bin/any.rhtml',
+        response, body = http.post( '/cgi-bin/any.rhtml',
                                 'querytype=subject&target=ruby' )
     }
 
@@ -54,12 +55,13 @@ methods of Net::HTTP but its instances always connect to
 proxy, instead of given host.
 
     require 'net/http'
+    Net::HTTP.version_1_1   # declear to use 1.1 features.
 
     $proxy_addr = 'your.proxy.addr'
     $proxy_port = 8080
           :
     Net::HTTP::Proxy($proxy_addr, $proxy_port).start( 'some.www.server' ) {|http|
-      # always connect to your.proxy.addr:8080
+        # always connect to your.proxy.addr:8080
           :
     }
 
@@ -69,7 +71,7 @@ there's no need to change code if there's proxy or not.
 === Redirect
 
     require 'net/http'
-    Net::HTTP.version_1_1
+    Net::HTTP.version_1_1   # declear to use 1.1 features.
 
     host = 'www.ruby-lang.org'
     path = '/'
@@ -92,16 +94,18 @@ URI class will be included in ruby standard library.
 === Basic Authentication
 
     require 'net/http'
+    Net::HTTP.version_1_1   # declear to use 1.1 features.
 
     Net::HTTP.start( 'auth.some.domain' ) {|http|
-        response , = http.get( '/need-auth.cgi',
-                'Authentication' => ["#{account}:#{password}"].pack('m').strip )
-        print response.body
+        response, body = http.get( '/need-auth.cgi',
+                'Authorization' => 'Basic ' + ["#{account}:#{password}"].pack('m').strip )
+        print body
     }
 
 In version 1.2 (Ruby 1.7 or later), you can write like this:
 
     require 'net/http'
+    Net::HTTP.version_1_2   # declear to use 1.2 features.
 
     req = Net::HTTP::Get.new('/need-auth.cgi')
     req.basic_auth 'account', 'password'
@@ -125,7 +129,7 @@ you to use 1.2 features again.
     Net::HTTP.version_1_2
     Net::HTTP.start {|http3| ...(http3 has 1.2 features)... }
 
-Yes, this is not thread-safe.
+This function is not thread-safe.
 
 == class Net::HTTP
 
@@ -213,21 +217,22 @@ Yes, this is not thread-safe.
 : proxy_port
     port number of proxy host. If self does not use a proxy, nil.
 
-: get( path, header = nil, dest = '' )
+: get( path, header = nil )
 : get( path, header = nil ) {|str| .... }
     gets data from PATH on the connecting host.
     HEADER must be a Hash like { 'Accept' => '*/*', ... }.
-    Response body is written into DEST by using "<<" method.
 
-    This method returns Net::HTTPResponse object.
+    In version 1.1, this method returns a pair of objects,
+    a Net::HTTPResponse object and entity body string.
+    In version 1.2, this method returns a Net::HTTPResponse
+    object.
 
-    If called with block, gives entity body little by little
-    to the block (as String).
+    If called with block, gives entity body string to the block
+    little by little.
 
     In version 1.1, this method might raises exception for also
     3xx (redirect). On the case you can get a HTTPResponse object
     by "anException.response".
-
     In version 1.2, this method never raises exception.
 
         # version 1.1 (bundled with Ruby 1.6)
@@ -246,10 +251,6 @@ Yes, this is not thread-safe.
               f.write str
             end
         }
-        # same effect
-        File.open( 'save.txt', 'w' ) {|f|
-            http.get '/~foo/', nil, f
-        }
 
 : head( path, header = nil )
     gets only header from PATH on the connecting host.
@@ -260,6 +261,7 @@ Yes, this is not thread-safe.
     In version 1.1, this method might raises exception for also
     3xx (redirect). On the case you can get a HTTPResponse object
     by "anException.response".
+    In version 1.2, this method never raises exception.
 
         response = nil
         Net::HTTP.start( 'some.www.server', 80 ) {|http|
@@ -267,67 +269,70 @@ Yes, this is not thread-safe.
         }
         p response['content-type']
 
-: post( path, data, header = nil, dest = '' )
+: post( path, data, header = nil )
 : post( path, data, header = nil ) {|str| .... }
-    posts "data" (must be String) to "path".
-    If the body exists, also gets entity body.
-    Response body is written into "dest" by using "<<" method.
-    "header" must be a Hash like { 'Accept' => '*/*', ... }.
-    This method returns Net::HTTPResponse object.
+    posts DATA (must be String) to PATH. HEADER must be a Hash
+    like { 'Accept' => '*/*', ... }.
+
+    In version 1.1, this method returns a pair of objects, a
+    Net::HTTPResponse object and an entity body string.
+    In version 1.2, this method returns a Net::HTTPReponse object.
 
     If called with block, gives a part of entity body string.
 
     In version 1.1, this method might raises exception for also
     3xx (redirect). On the case you can get a HTTPResponse object
     by "anException.response".
+    In version 1.2, this method never raises exception.
 
         # version 1.1
-        response, body = http.post( '/cgi-bin/search.rb', 'querytype=subject&target=ruby' )
+        response, body = http.post( '/cgi-bin/search.rb', 'query=subject&target=ruby' )
+
         # version 1.2
-        response = http.post( '/cgi-bin/search.rb', 'querytype=subject&target=ruby' )
-        # compatible for both version
-        response , = http.post( '/cgi-bin/search.rb', 'querytype=subject&target=ruby' )
+        response = http.post( '/cgi-bin/search.rb', 'query=subject&target=ruby' )
+
+        # compatible in both version
+        response , = http.post( '/cgi-bin/search.rb', 'query=subject&target=ruby' )
 
         # using block
         File.open( 'save.html', 'w' ) {|f|
-            http.post( '/cgi-bin/search.rb', 'querytype=subject&target=ruby' ) do |str|
+            http.post( '/cgi-bin/search.rb',
+                       'query=subject&target=ruby' ) do |str|
               f.write str
             end
         }
-        # same effect
-        File.open( 'save.html', 'w' ) {|f|
-            http.post '/cgi-bin/search.rb', 'querytype=subject&target=ruby', nil, f
-        }
 
-: get2( path, header = nil )
-: get2( path, header = nil ) {|response| .... }
+: request_get( path, header = nil )
+: request_get( path, header = nil ) {|response| .... }
     gets entity from PATH. This method returns a HTTPResponse object.
 
     When called with block, keep connection while block is executed
     and gives a HTTPResponse object to the block.
 
-    This method never raise any ProtocolErrors.
+    This method never raises Net::* exceptions.
 
         # example
-        response = http.get2( '/index.html' )
+        response = http.request_get( '/index.html' )
         p response['content-type']
         puts response.body          # body is already read
 
         # using block
-        http.get2( '/index.html' ) {|response|
+        http.request_get( '/index.html' ) {|response|
             p response['content-type']
             response.read_body do |str|   # read body now
               print str
             end
         }
 
-: post2( path, header = nil )
-: post2( path, header = nil ) {|response| .... }
+: request_post( path, data, header = nil )
+: request_post( path, data, header = nil ) {|response| .... }
     posts data to PATH. This method returns a HTTPResponse object.
 
     When called with block, gives a HTTPResponse object to the block
     before reading entity body, with keeping connection.
 
+    This method never raises Net::* exceptions.
+
         # example
         response = http.post2( '/cgi-bin/nice.rb', 'datadatadata...' )
         p response.status
@@ -403,127 +408,14 @@ module Net
 
     HTTPVersion = '1.1'
 
-    #
-    # connection
-    #
-
     protocol_param :port, '80'
 
-    def initialize( addr, port = nil )
-      super
-
-      @curr_http_version = HTTPVersion
-      @seems_1_0_server = false
-    end
-
-    private
-
-    def conn_command( sock )
-    end
-
-    def do_finish
-    end
-
-
-    #
-    # proxy
-    #
-
-    public
-
-
-    class << self
-
-      def Proxy( p_addr, p_port = nil )
-        if p_addr then
-          ProxyMod.create_proxy_class( p_addr, p_port || self.port )
-        else
-          self
-        end
-      end
-
-      alias orig_new new
-
-      def new( address, port = nil, p_addr = nil, p_port = nil )
-        c = p_addr ? self::Proxy(p_addr, p_port) : self
-        i = c.orig_new( address, port )
-        setvar i
-        i
-      end
-
-      def start( address, port = nil, p_addr = nil, p_port = nil, &block )
-        new( address, port, p_addr, p_port ).start( &block )
-      end
-
-      @is_proxy_class = false
-      @proxy_addr = nil
-      @proxy_port = nil
-
-      def proxy_class?
-        @is_proxy_class
-      end
-
-      attr_reader :proxy_address
-      attr_reader :proxy_port
-
-    end
-
-    def proxy?
-      type.proxy_class?
-    end
-
-    def proxy_address
-      type.proxy_address
-    end
-
-    def proxy_port
-      type.proxy_port
-    end
-
-    alias proxyaddr proxy_address
-    alias proxyport proxy_port
-
-    def edit_path( path )
-      path
-    end
-
-
-    module ProxyMod
-
-      def self.create_proxy_class( p_addr, p_port )
-        mod = self
-        klass = Class.new( HTTP )
-        klass.module_eval {
-          include mod
-          @is_proxy_class = true
-          @proxy_address = p_addr
-          @proxy_port    = p_port
-        }
-        klass
-      end
-
-      private
-    
-      def conn_socket( addr, port )
-        super proxy_address, proxy_port
-      end
-
-      def edit_path( path )
-        'http://' + addr_port + path
-      end
-    
-    end   # module ProxyMod
-
 
     #
     # for backward compatibility
     #
 
-    if Version < '1.2.0' then   ###noupdate
-      @@newimpl = false
-    else
-      @@newimpl = true
-    end
+    @@newimpl = false
 
     class << self
 
@@ -541,7 +433,7 @@ module Net
 
       private
 
-      def setvar( obj )
+      def setimplversion( obj )
         f = @@newimpl
         obj.instance_eval { @newimpl = f }
       end
@@ -549,91 +441,265 @@ module Net
     end
 
 
+    #
+    # short cut methods
+    #
+
+    def HTTP.get( addr, path, port = nil )
+      req = Get.new( path )
+      resp = nil
+      new( addr, port || HTTP.port ).start {|http|
+          resp = http.request( req )
+      }
+      resp.body
+    end
+
+    def HTTP.get_print( addr, path, port = nil )
+      new( addr, port || HTTP.port ).start {|http|
+          http.get path, nil, $stdout
+      }
+      nil
+    end
+
+
+    #
+    # constructors
+    #
+
+    class << self
+
+      def start( address, port = nil, p_addr = nil, p_port = nil, &block )
+        new( address, port, p_addr, p_port ).start( &block )
+      end
+
+      alias newobj new
+
+      def new( address, port = nil, p_addr = nil, p_port = nil )
+        obj = Proxy(p_addr, p_port).newobj(address, port)
+        setimplversion obj
+        obj
+      end
+
+    end
+
+    def initialize( addr, port = nil )
+      super
+      @curr_http_version = HTTPVersion
+      @seems_1_0_server = false
+    end
+
+
+    #
+    # connection
+    #
+
+    private
+
+    def do_start
+      conn_socket
+    end
+
+    def do_finish
+      disconn_socket
+    end
+
+
+    #
+    # proxy
+    #
+
+    public
+
+    # without proxy
+    @is_proxy_class = false
+    @proxy_addr = nil
+    @proxy_port = nil
+
+    class << self
+      def Proxy( p_addr, p_port = nil )
+        p_addr or return self
+
+        p_port ||= port()
+        mod = ProxyDelta
+        proxyclass = Class.new(self)
+        proxyclass.module_eval {
+            include mod
+            # with proxy
+            @is_proxy_class = true
+            @proxy_address = p_addr
+            @proxy_port    = p_port
+        }
+        proxyclass
+      end
+
+      def proxy_class?
+        @is_proxy_class
+      end
+
+      attr_reader :proxy_address
+      attr_reader :proxy_port
+    end
+
+    def proxy?
+      type.proxy_class?
+    end
+
+    def proxy_address
+      type.proxy_address
+    end
+
+    def proxy_port
+      type.proxy_port
+    end
+
+    alias proxyaddr proxy_address
+    alias proxyport proxy_port
+
+    private
+
+    # without proxy
+
+    def conn_address
+      address
+    end
+
+    def conn_port
+      port
+    end
+
+    def edit_path( path )
+      path
+    end
+
+    module ProxyDelta
+      private
+
+      # with proxy
+    
+      def conn_address
+        proxy_address
+      end
+
+      def conn_port
+        proxy_port
+      end
+
+      def edit_path( path )
+        'http://' + addr_port() + path
+      end
+    end
+
+
     #
     # http operations
     #
 
     public
 
-    def self.define_http_method_interface( nm, hasdest, hasdata )
-      name = nm.id2name.downcase
-      cname = nm.id2name
-      lineno = __LINE__ + 2
-      src = <<"      ----"
+    def get( path, initheader = nil, dest = nil, &block )
+      res = nil
+      request( Get.new(path,initheader) ) {|res|
+          res.read_body dest, &block
+      }
+      unless @newimpl then
+        res.value
+        return res, res.body
+      end
 
-        def #{name}( path, #{hasdata ? 'data,' : ''}
-                     u_header = nil #{hasdest ? ',dest = nil, &block' : ''} )
-          resp = nil
-          request(
-              #{cname}.new( path, u_header ) #{hasdata ? ',data' : ''}
-          ) do |resp|
-            resp.read_body( #{hasdest ? 'dest, &block' : ''} )
-          end
-          if @newimpl then
-            resp
-          else
-            resp.value
-            #{hasdest ? 'return resp, resp.body' : 'resp'}
-          end
-        end
-
-        def #{name}2( path, #{hasdata ? 'data,' : ''}
-                      u_header = nil, &block )
-          request( #{cname}.new(path, u_header),
-                   #{hasdata ? 'data,' : ''} &block )
-        end
-      ----
-      module_eval src, __FILE__, lineno
+      res
     end
 
-    define_http_method_interface :Get,  true,  false
-    define_http_method_interface :Head, false, false
-    define_http_method_interface :Post, true,  true
-    define_http_method_interface :Put,  false, true
+    def head( path, initheader = nil )
+      res = request( Head.new(path,initheader) )
+      @newimpl or res.value
+      res
+    end
+
+    def post( path, data, initheader = nil, dest = nil, &block )
+      res = nil
+      request( Post.new(path,initheader), data ) {|res|
+          res.read_body dest, &block
+      }
+      unless @newimpl then
+        res.value
+        return res, res.body
+      end
+
+      res
+    end
+
+    def put( path, data, initheader = nil )
+      res = request( Put.new(path,initheader), data )
+      @newimpl or res.value
+      res
+    end
+
+    def request_get( path, initheader = nil, &block )
+      request Get.new(path,initheader), &block
+    end
+
+    def request_head( path, initheader = nil, &block )
+      request Head.new(path,initheader), &block
+    end
+
+    def request_post( path, data, initheader = nil, &block )
+      request Post.new(path,initheader), data, &block
+    end
+
+    def request_put( path, data, initheader = nil, &block )
+      request Put.new(path,initheader), data, &block
+    end
+
+    alias get2   request_get
+    alias head2  request_head
+    alias post2  request_post
+    alias put2   request_put
+
+    def send_request( name, path, body = nil, header = nil )
+      r = HTTPGenericRequest.new( name, (body ? true : false), true,
+                                  path, header )
+      request r, body
+    end
 
     def request( req, body = nil, &block )
       unless active? then
         start {
-          req['connection'] = 'close'
-          return request(req, body, &block)
+            req['connection'] = 'close'
+            return request(req, body, &block)
         }
       end
         
-      connecting( req ) {
-        req.__send__( :exec,
-                @socket, @curr_http_version, edit_path(req.path), body )
-        yield req.response if block_given?
-      }
-      req.response
-    end
+      begin_transport(req)
+          req.__send__(:exec,
+                       @socket, @curr_http_version, edit_path(req.path), body)
+          begin
+            res = HTTPResponse.read_new(@socket, req.response_body_permitted?)
+          end while ContinueCode === res
+          yield res if block_given?
+      end_transport(req, res)
 
-    def send_request( name, path, body = nil, header = nil )
-      r = ::Net::NetPrivate::HTTPGenericRequest.new(
-              name, (body ? true : false), true,
-              path, header )
-      request r, body
+      res
     end
 
-
     private
 
-
-    def connecting( req )
+    def begin_transport( req )
       if @socket.closed? then
-        re_connect
+        reconn_socket
       end
       if not req.body_exist? or @seems_1_0_server then
         req['connection'] = 'close'
       end
       req['host'] = addr_port()
+    end
 
-      yield req
-      req.response.__send__ :terminate
-      @curr_http_version = req.response.http_version
+    def end_transport( req, res )
+      res.__send__ :terminate
+      @curr_http_version = res.http_version
 
-      if not req.response.body then
+      if not res.body then
         @socket.close
-      elsif keep_alive? req, req.response then
+      elsif keep_alive? req, res then
         D 'Conn keep-alive'
         if @socket.closed? then   # (only) read stream had been closed
           D 'Conn (but seems 1.0 server)'
@@ -664,25 +730,6 @@ module Net
     # utils
     #
 
-    public
-
-    def self.get( addr, path, port = nil )
-      req = Get.new( path )
-      resp = nil
-      new( addr, port || HTTP.port ).start {|http|
-        resp = http.request( req )
-      }
-      resp.body
-    end
-
-    def self.get_print( addr, path, port = nil )
-      new( addr, port || HTTP.port ).start {|http|
-        http.get path, nil, $stdout
-      }
-      nil
-    end
-
-
     private
 
     def addr_port
@@ -701,7 +748,6 @@ module Net
   HTTPSession = HTTP
 
 
-
   class Code
 
     def http_mkchild( bodyexist = nil )
@@ -771,8 +817,6 @@ module Net
   ### header
   ###
 
-  net_private {
-
   module HTTPHeader
 
     def size
@@ -789,10 +833,12 @@ module Net
       @header[ key.downcase ] = val
     end
 
-    def each( &block )
+    def each_header( &block )
       @header.each( &block )
     end
 
+    alias each each_header
+
     def each_key( &block )
       @header.each_key( &block )
     end
@@ -846,9 +892,7 @@ module Net
     end
 
     def range=( r, fin = nil )
-      if fin then
-        r = r ... r+fin
-      end
+      r = (r ... r + fin) if fin
 
       case r
       when Numeric
@@ -920,9 +964,9 @@ module Net
 
   class HTTPGenericRequest
 
-    include ::Net::NetPrivate::HTTPHeader
+    include HTTPHeader
 
-    def initialize( m, reqbody, resbody, path, uhead = nil )
+    def initialize( m, reqbody, resbody, path, initheader = nil )
       @method = m
       @request_has_body = reqbody
       @response_has_body = resbody
@@ -930,8 +974,8 @@ module Net
       @response = nil
 
       @header = tmp = {}
-      return unless uhead
-      uhead.each do |k,v|
+      return unless initheader
+      initheader.each do |k,v|
         key = k.downcase
         if tmp.key? key then
           $stderr.puts "WARNING: duplicated HTTP header: #{k}" if $VERBOSE
@@ -966,43 +1010,17 @@ module Net
     # write
     #
 
-    def exec( sock, ver, path, body, &block )
+    def exec( sock, ver, path, body )
       if body then
-        check_body_premitted
-        check_arg_b body, block
-        sendreq_with_body sock, ver, path, body, &block
+        raise ArgumentError, 'HTTP request body is not premitted'\
+                                        unless request_body_permitted?
+        send_request_with_body sock, ver, path, body
       else
-        check_arg_n body
-        sendreq_no_body sock, ver, path
-      end
-      @response = r = get_response( sock )
-      r
-    end
-
-    def check_body_premitted
-      request_body_permitted? or
-          raise ArgumentError, 'HTTP request body is not premitted'
-    end
-
-    def check_arg_b( data, block )
-      if data and block then
-        raise ArgumentError, 'both of data and block given'
-      end
-      unless data or block then
-        raise ArgumentError, 'str or block required'
+        request sock, ver, path
       end
     end
 
-    def check_arg_n( data )
-      data and raise ArgumentError, "data is not permitted for #{@method}"
-    end
-
-
-    def sendreq_no_body( sock, ver, path )
-      request sock, ver, path
-    end
-
-    def sendreq_with_body( sock, ver, path, body )
+    def send_request_with_body( sock, ver, path, body )
       if block_given? then
         ac = Accumulator.new
         yield ac              # must be yield, DO NOT USE block.call
@@ -1013,6 +1031,11 @@ module Net
       @header['content-length'] = data.size.to_s
       @header.delete 'transfer-encoding'
 
+      unless @header['content-type'] then
+        $stderr.puts 'Content-Type did not set; using application/x-www-form-urlencoded' if $VERBOSE
+        @header['content-type'] = 'application/x-www-form-urlencoded'
+      end
+
       request sock, ver, path
       sock.write data
     end
@@ -1024,81 +1047,43 @@ module Net
       end
       sock.writeline ''
     end
-
-    #
-    # read
-    #
-
-    def get_response( sock )
-      begin
-        resp = ::Net::NetPrivate::HTTPResponse.new_from_socket(sock,
-                                                response_body_permitted?)
-      end while ContinueCode === resp
-      resp
-    end
   
   end
 
 
   class HTTPRequest < HTTPGenericRequest
 
-    def initialize( path, uhead = nil )
+    def initialize( path, initheader = nil )
       super type::METHOD,
             type::REQUEST_HAS_BODY,
             type::RESPONSE_HAS_BODY,
-            path, uhead
+            path, initheader
     end
 
   end
 
 
-  class Accumulator
-  
-    def initialize
-      @buf = ''
-    end
-
-    def write( s )
-      @buf.concat s
-    end
-
-    def <<( s )
-      @buf.concat s
-      self
-    end
-
-    def terminate
-      ret = @buf
-      @buf = nil
-      ret
-    end
-  
-  end
-
-  }
-
-
   class HTTP
 
-    class Get < ::Net::NetPrivate::HTTPRequest
+    class Get < HTTPRequest
       METHOD = 'GET'
       REQUEST_HAS_BODY  = false
       RESPONSE_HAS_BODY = true
     end
 
-    class Head < ::Net::NetPrivate::HTTPRequest
+    class Head < HTTPRequest
       METHOD = 'HEAD'
       REQUEST_HAS_BODY = false
       RESPONSE_HAS_BODY = false
     end
 
-    class Post < ::Net::NetPrivate::HTTPRequest
+    class Post < HTTPRequest
       METHOD = 'POST'
       REQUEST_HAS_BODY = true
       RESPONSE_HAS_BODY = true
     end
 
-    class Put < ::Net::NetPrivate::HTTPRequest
+    class Put < HTTPRequest
       METHOD = 'PUT'
       REQUEST_HAS_BODY = true
       RESPONSE_HAS_BODY = true
@@ -1112,11 +1097,9 @@ module Net
   ### response
   ###
 
-  net_private {
-
   class HTTPResponse < Response
 
-    include ::Net::NetPrivate::HTTPHeader
+    include HTTPHeader
 
     CODE_CLASS_TO_OBJ = {
       '1' => HTTPInformationCode,
@@ -1173,37 +1156,38 @@ module Net
 
     class << self
 
-      def new_from_socket( sock, hasbody )
-        resp = readnew( sock, hasbody )
-
-        while true do
-          line = sock.readuntil( "\n", true )   # ignore EOF
-          line.sub!( /\s+\z/, '' )                 # don't use chop!
-          break if line.empty?
-
-          m = /\A([^:]+):\s*/.match( line )
-          m or raise HTTPBadResponse, 'wrong header line format'
-          nm = m[1]
-          line = m.post_match
-          if resp.key? nm then
-            resp[nm] << ', ' << line
+      def read_new( sock, hasbody )
+        httpv, code, msg = read_status(sock)
+        res = new(code, msg, sock, hasbody, httpv)
+        each_response_header(sock) do |k,v|
+          if res.key? k then
+            res[k] << ', ' << v
           else
-            resp[nm] = line
+            res[k] = v
           end
         end
-
-        resp
+        res
       end
 
       private
 
-      def readnew( sock, hasbody )
+      def read_status( sock )
         str = sock.readline
         m = /\AHTTP(?:\/(\d+\.\d+))?\s+(\d\d\d)\s*(.*)\z/in.match( str )
         m or raise HTTPBadResponse, "wrong status line: #{str}"
-        discard, httpv, stat, desc = *m.to_a
-        
-        new( stat, desc, sock, hasbody, httpv )
+        m.to_a[1,3]
+      end
+
+      def each_response_header( sock )
+        while true do
+          line = sock.readuntil( "\n", true )   # ignore EOF
+          line.sub!( /\s+\z/, '' )              # don't use chop!
+          break if line.empty?
+
+          m = /\A([^:]+):\s*/.match(line) or
+                  raise HTTPBadResponse, 'wrong header line format'
+          yield m[1], m.post_match
+        end
       end
 
     end
@@ -1250,22 +1234,22 @@ module Net
     #
 
     def read_body( dest = nil, &block )
-      if @read and (dest or block) then
-        raise IOError, "#{type}\#read_body called twice with argument"
+      if @read then
+        (dest or block) and
+                raise IOError, "#{type}\#read_body called twice with argument"
+        return @body
       end
 
-      unless @read then
-        to = procdest( dest, block )
-        stream_check
+      to = procdest(dest, block)
+      stream_check
 
-        if @body_exist and code_type.body_exist? then
-          read_body_0 to
-          @body = to
-        else
-          @body = nil
-        end
-        @read = true
+      if @body_exist and code_type.body_exist? then
+        read_body_0 to
+        @body = to
+      else
+        @body = nil
       end
+      @read = true
 
       @body
     end
@@ -1273,10 +1257,8 @@ module Net
     alias body read_body
     alias entity read_body
 
-
     private
 
-
     def terminate
       read_body
     end
@@ -1305,11 +1287,11 @@ module Net
 
       while true do
         line = @socket.readline
-        m = /[0-9a-fA-F]+/.match( line )
+        m = /[0-9a-fA-F]+/.match(line)
         m or raise HTTPBadResponse, "wrong chunk size line: #{line}"
         len = m[0].hex
         break if len == 0
-        @socket.read( len, dest ); total += len
+        @socket.read len, dest; total += len
         @socket.read 2   # \r\n
       end
       until @socket.readline.empty? do
@@ -1326,7 +1308,7 @@ module Net
         raise ArgumentError, 'both of arg and block are given for HTTP method'
       end
       if block then
-        ::Net::NetPrivate::ReadAdapter.new block
+        ReadAdapter.new block
       else
         dest || ''
       end
@@ -1334,10 +1316,14 @@ module Net
 
   end
 
-  }
 
-
-  HTTPResponse         = NetPrivate::HTTPResponse
-  HTTPResponseReceiver = NetPrivate::HTTPResponse
+  # for backward compatibility
+  module NetPrivate
+    HTTPResponse         = ::Net::HTTPResponse
+    HTTPGenericRequest   = ::Net::HTTPGenericRequest
+    HTTPRequest          = ::Net::HTTPRequest
+    HTTPHeader           = ::Net::HTTPHeader
+  end
+  HTTPResponceReceiver = HTTPResponse
 
 end   # module Net
